# -*- coding: utf-8 -*-

import logging

from odoo import api, fields, models
from odoo import SUPERUSER_ID
from odoo.http import request
from odoo.exceptions import UserError
from odoo.addons.phone_validation.tools import phone_validation

from dateutil.relativedelta import relativedelta

_logger = logging.getLogger(__name__)


class Lead(models.Model):

    _inherit = 'crm.lead'

    existing_subscription_ids = fields.One2many('sale.order', compute='_compute_existing_subscription_ids',)

    @api.depends('email_domain_criterion')
    def _compute_existing_subscription_ids(self):
        for lead in self:
            if lead.email_domain_criterion:
                today = fields.Date.context_today(self)
                domain = [
                    ('subscription_state', 'in', ['3_progress', '4_paused', '6_churn']),
                    '|',
                    ('end_date', '>', today - relativedelta(months=6)),
                    ('end_date', '=', False),
                ]
                lead.existing_subscription_ids = self.env['sale.order']._check_duplicate(lead.email_normalized, domain)
            else:
                lead.existing_subscription_ids = False

    def action_existing_subscritions(self):
        return {
            'name': 'Subscriptions',
            'res_model': 'sale.order',
            'views': [
                [self.env.ref('sale_subscription.sale_subscription_view_tree').id, 'tree'],
                [self.env.ref('sale_subscription.sale_subscription_primary_form_view').id, "form"]],
            'domain': [('id', 'in', self.existing_subscription_ids.ids)],
            'search_view_id': self.env.ref('sale_subscription.sale_subscription_view_search').id,
            'type': 'ir.actions.act_window',
        }

    def action_new_quotation(self):
        action = super().action_new_quotation()
        # create the quotation using the current user company and avoid multi-company issues
        action['context']['default_company_id'] = self.env.company.id
        return action

    def get_score_domain_cookies(self):
        # cookies for *.odoo.com
        # Need to be chared between accounts and wwww
        dom = "." + ".".join(request.httprequest.host.rsplit(".", 2)[1:])
        if ':' in dom:  # Hack if in local mode with port
            dom = request.httprequest.host
        return dom

    def get_tag_id(self, name):
        """ Given a label, return the corresponding tag id, or create a new one """
        Tag = self.env['crm.tag'].sudo()
        tag = Tag.search([('name', '=', name)], limit=1)
        if not tag:
            tag = Tag.create({'name': name})
            _logger.info('Creating tag for lead: %s', name)
        return tag.id

    def create(self, vals_list):
        result = super().create(vals_list)
        result.filtered(lambda l: l.existing_subscription_ids).write({
            'tag_ids': [(4, self.get_tag_id('Already Customer'))],
        })
        return result

    def create_lead(self, text=None, email=None, partner_id=None, scope=None, plan=None, country_id=None, contact_name=None, phone=None, company=None, channel=None, location=None):
        supported_company_size = ["1-5", "5-20", "20-50", "50-250", "250-over"]

        if not email and not partner_id:
            raise UserError("Error :Lead creation require at least a partner_id or an email.")

        lead_values = self._context.get('all_lead_values', {}) or {}
        name = contact_name
        if partner_id:
            partner = self.env['res.partner'].browse(partner_id)
            if partner.exists():
                name = partner.name or partner.email or 'unknow partner'
            # # copy tag from partner
            # for tag in partner.category_id:
            #     lead_values.setdefault('tag_ids', []).extend([(4, self.get_tag_id(tag.name))])
        else:
            partner = None

        company_size = self._context.get('all_lead_kwargs', {}).get('company_size') or False
        lang_code = self._context.get('all_lead_kwargs', {}).get('lang') or False
        if lang_code:
            lang = self.env['res.lang'].with_context(active_test=False).search([('code', '=', lang_code)], limit=1)
            lead_values.update({'lang_id': lang.id})
        # if not company_size but already a partner, we can maybe retrieve the company size in the categs, we need it to check if stage is "50+"
        if company_size not in supported_company_size and partner:
            company_size = [tag.name[5:] for tag in partner.category_id if tag.name.startswith("Size-") and tag.name[5:] in supported_company_size]

        if company_size in supported_company_size:
            lead_values.setdefault('tag_ids', []).extend([(4, self.get_tag_id('Size-%s' % company_size))])

        # ADD TAG PLAN+
        if plan and plan in ['plan_to_use', 'plan_to_sell', 'plan_to_test', 'plan_to_test_student', 'plan_to_test_teacher']:
            # plan = plan in ['plan_to_test_student',  'plan_to_test_teacher'] and 'plan_to_test' or plan
            lead_values.setdefault('tag_ids', []).extend([(4, self.get_tag_id(plan))])

        if scope == 'odoo_sh_question':
            lead_values.setdefault('tag_ids', []).extend([(4, self.get_tag_id('OdooSH_VIP'))])

        if email:
            lead_values.update({'email_from': email})
        if text:
            lead_values.update({'description': text})
        if contact_name:
            lead_values.update({'contact_name': contact_name})
        if company:
            lead_values.update({'partner_name': company})
            name += ' (%s)' % company
        if phone:
            lead_values.update({'phone': phone})

        if len(lead_values.get('phone') or '') < 5:
            lead_values['phone'] = False

        if country_id and not lead_values.get('country_id'):
            lead_values['country_id'] = country_id

        if lead_values.get("country_id") and lead_values.get("phone"):
            country = self.env["res.country"].browse(lead_values["country_id"])
            lead_values["phone"] = phone_validation.phone_format(
                lead_values["phone"], country.code, country.phone_code,
                raise_exception=False,
            )

        if location:
            for key in ['partner_latitude', 'partner_longitude', 'city', 'state_id']:
                if location.get(key):
                    lead_values[key] = location[key]

        title = (scope == "x_Download" and "'s Download") or (scope == "Question" and "'s Question") or \
                (plan and plan == 'plan_to_sell' and ' Partnership' or ' Registration') or \
                (scope == "odoo_sh_question" and 'OdooSH_VIP') or ''
        lead_values.update({
            'name': '%s%s' % (name or email, title),
            'partner_id': partner_id,
            'user_id': False,
            'team_id': False,
            'referred': scope,
        })
        for bad_key in set(lead_values.keys()) - set(self._fields.keys()):
            lead_values.pop(bad_key, None)
        return self.with_user(SUPERUSER_ID).with_context(mail_create_nosubscribe=True).create(lead_values)

    def _prepare_customer_values(self, partner_name, is_company=False, parent_id=False):
        vals = super()._prepare_customer_values(partner_name, is_company=is_company, parent_id=parent_id)
        if self.lang_id and not self.lang_id.active:
            # lang is not available or enabled, get the nearest matching
            # language instead, for ex. "ar_001" is enabled as language for
            # /trial but disabled on the website/backend, use "ar" instead
            lang_codes = [code for code, _name in self.env["res.lang"].get_installed()]
            lang_short = self.lang_id.code.partition('_')[0].partition('@')[0]
            for code in lang_codes:
                if code.startswith(lang_short):
                    vals["lang"] = code
                    break
            else:
                # in case no lang matches do not set it at all
                vals.pop("lang", None)
        return vals
